-- Parsec
-- Module 'drawing'
require "sprites"
require "lib/core"
require "lib/string"

drawing = {
  fontcache = {},
  spritecache = {}

}
const = {
  font = "sans.ttf",
  fontb = "sans-b.ttf",
  fontem = "sans-em.ttf",
  fonti = "sans-i.ttf",
  fontsize = 12,
  textcolor = "#ADADAD"
}

function drawing.init()
  --clear all cache
  for k,v in ipairs(drawing.fontcache) do
    sprite.free_font(v);
    drawing.fontcache[k] = nil;
  end
  for k,v in ipairs(drawing.spritecache) do
    sprite.free(v);
    drawing.spritecache[k] = nil;
  end
  drawing.fontcache={};
  drawing.spritecache={};
end

function drawing.fmt_text(s)
  s = string.gsub(s, ' "', " ")
  s = string.gsub(s, '"', "")
  return s;
end

function drawing.font(name, size)
  if name == nil then
    name = const.font;
  end
  if size == nil then
    size = const.fontsize;
  end
  local key = name..tostring(size);
  local f = drawing.fontcache[key];
  if f == nil then
    f = sprite.font("theme/"..name, size);
    drawing.fontcache[key] = f;
  end
  return f;
end

function drawing.text_height(fn, fs)
  local f = drawing.font(fn, fs);
  return math.ceil(sprite.font_scaled_size(sprite.font_height(f)));
end

function drawing.text_width(txt, fn, fs)
  local f = drawing.font(fn, fs);
  return math.ceil(sprite.font_scaled_size(sprite.text_size(f, txt)));
end

function drawing.text_bounds(txt, width, fn, fs)
  local function calc_rows(txt)
    if txt == nil then
      return 0;
    end  
    local ln = txt:lines();
    local rows = 0;
    for i,e in pairs(ln) do
      if e == "" then
        rows = rows + 1;
      else
        local rl = drawing.text_width(e, fn, fs);
        local n = math.ceil(rl / width);
        rows = rows + n;
      end
    end    
    return rows;
  end
  local r = calc_rows(txt,fn);
  local fh = drawing.text_height(fn, fs);
  return fh*r;
end

function drawing.sprite_blank(w, h, cache)
  if cache then
    local key = w.."+"..h;
    local spr = drawing.spritecache[key];
    if spr == nil then
      spr = sprite.blank(w, h);
      drawing.spritecache[key] = spr;
    end
    return spr;
  else
    return sprite.blank(w, h);
  end
end

function drawing.box(w, h, c, o)
  local key = w.."+"..h.."+"..c.."+"..o;
  local spr = drawing.spritecache[key];
  if spr == nil then
    spr = sprite.box(w, h, c, o);
    drawing.spritecache[key] = spr;
  end
  return spr;
end

function drawing.sprite(file)
  local spr = drawing.spritecache[file];
  if spr == nil then
    spr = sprite.load(file);
    drawing.spritecache[file] = spr;
  end
  return spr;
end

function drawing.clean(pic, x, y, w, h)
  notnil(w, "w");
  notnil(h, "h");
  local pt = drawing.sprite_blank(w, h);
  sprite.copy(pt, pic, x, y);
  sprite.free(pt);
end

function drawing.copy(src, dst, x, y, alpha)
  local w,h = sprite.size(src);
  sprite.compose(src, 0, 0, w, h, dst, x, y, alpha);
end

function drawing.render_text(copy, pic, txt, x, y, c, fn, fs, alpha)
  txt = drawing.fmt_text(txt);
  if c == nil then
    c = const.textcolor;
  end
  local f = drawing.font(fn, fs);
  local t = sprite.text(f, txt, c);
  if copy then
    drawing.copy(t, pic, x, y, alpha);
  else
    sprite.draw(t, pic, x, y);
  end
  sprite.free(t);
end

function drawing.draw_text(pic, txt, x, y, c, fn, fs)
  drawing.render_text(false, pic, txt, x, y, c, fn, fs);
end

function drawing.draw_text_center(pic, txt, w, y, c, fn, fs, copy)
  local f = drawing.font(fn, fs);
  local t = sprite.text(f, txt, c);
  local tw = sprite.size(t);
  local x = (w - tw) / 2;
  sprite.free(t);
  drawing.render_text(copy, pic, txt, x, y, c, fn, fs);
  return x+tw;
end

function drawing.copy_text(pic, txt, x, y, c, fn, fs, alpha)
  drawing.render_text(true, pic, txt, x, y, c, fn, fs, alpha);
end

function drawing.load_pic(file, cache)
  if not cache then
    return sprite.load(file);
  else
    return drawing.sprite(file);
  end
end

function drawing.draw_pic(pic, file, x, y, cache)
  local p = drawing.load_pic(file, cache);
  sprite.draw(p, pic, x, y);
  if not cache then
    sprite.free(p);
  end
  return pic;
end

function drawing.copy_pic(pic, file, x, y, cache, alpha)
  local p = drawing.load_pic(file, cache);
  drawing.copy(p, pic, x, y, alpha);
  if not cache then
    sprite.free(p);
  end
  return pic;
end
